#include "decoder.h"

#include <zxing/common/GlobalHistogramBinarizer.h>
#include <zxing/Binarizer.h>
#include <zxing/BinaryBitmap.h>
#include <zxing/MultiFormatReader.h>
#include <zxing/DecodeHints.h>
#include "CameraImageWrapper.h"
#include "imagehandler.h"

#include <QtCore>
#include <qglobal.h>
#include <QQuickImageProvider>

using namespace zxing;

Decoder::Decoder(QObject *parent) :
    QObject(parent)
{
    decoder = new MultiFormatReader();
    setDecoder(DecoderFormat_QR_CODE |
               DecoderFormat_DATA_MATRIX |
               DecoderFormat_UPC_E |
               DecoderFormat_UPC_A |
               DecoderFormat_EAN_8 |
               DecoderFormat_EAN_13 |
               DecoderFormat_CODE_128 |
               DecoderFormat_CODE_39 |
               DecoderFormat_ITF |
               DecoderFormat_Aztec);
    imageHandler = new ImageHandler();
}

QString Decoder::decodeImage(QImage image)
{
    Ref<Result> result;
    emit decodingStarted();

    try {
        Ref<LuminanceSource> source(new CameraImageWrapper(image));

        Ref<Binarizer> binarizer;
        binarizer = new GlobalHistogramBinarizer(source);

        Ref<BinaryBitmap> binary(new BinaryBitmap(binarizer));

        // DecodeHints hints(DecodeHints::DEFAULT_HINT);
        DecodeHints hints((int)supportedFormats);

        //image.save("/tmp/saved.jpg");
        qDebug() << "[decodeImage()] About to start decoding";
        result = ((MultiFormatReader*)decoder)->decode(binary, hints);
        qDebug() << "[decodeImage()] Decoding finished";

        QString string = QString(result->getText()->getText().c_str());
        emit tagFound(string);
        emit decodingFinished(true);
        return string;
    }
    catch(zxing::Exception& e)
    {
        qDebug() << "[decodeImage()] Exception:" << e.what();
       emit decodingFinished(false);
       return "";
    }
}

QString Decoder::decodeImageQML(const QUrl &item)
{
    qDebug() << item;
    qDebug() << item.path();

    QQmlEngine * engine;
    QQuickImageProvider *imageProvider = 0;
    imageProvider = engine->imageProvider("camera");

    QSize size(300, 300);

    QImage image = imageProvider->requestImage("preview_1", new QSize(size.width(),size.height()), size);

    image.save("/tmp/preview.jpg");

    return Decoder::decodeImage(image);
}

void Decoder::setDecoder(DecoderFormatType hint)
{
    DecodeHints newHints;

    if(hint & DecoderFormat_QR_CODE)
        newHints.addFormat((BarcodeFormat)BarcodeFormat_QR_CODE);

    if(hint & DecoderFormat_DATA_MATRIX)
        newHints.addFormat((BarcodeFormat)BarcodeFormat_DATA_MATRIX);

    if(hint & DecoderFormat_UPC_E)
        newHints.addFormat((BarcodeFormat)BarcodeFormat_UPC_E);

    if(hint & DecoderFormat_UPC_A)
        newHints.addFormat((BarcodeFormat)BarcodeFormat_UPC_A);

    if(hint & DecoderFormat_EAN_8)
        newHints.addFormat((BarcodeFormat)BarcodeFormat_EAN_8);

    if(hint & DecoderFormat_EAN_13)
        newHints.addFormat((BarcodeFormat)BarcodeFormat_EAN_13);

    if(hint & DecoderFormat_CODE_128)
        newHints.addFormat((BarcodeFormat)BarcodeFormat_CODE_128);

    if(hint & DecoderFormat_CODE_39)
        newHints.addFormat((BarcodeFormat)BarcodeFormat_CODE_39);

    if(hint & DecoderFormat_ITF)
        newHints.addFormat((BarcodeFormat)BarcodeFormat_ITF);

    if(hint & DecoderFormat_Aztec)
        newHints.addFormat((BarcodeFormat)BarcodeFormat_AZTEC);

    supportedFormats = newHints.getCurrentHint();
}
